from enum import Enum


class SqlFunction(Enum):

    Length = "LENGTH"
    Replace = "REPLACE"
    Soundex = "SOUNDEX"
    Substring = "SUBSTRING"
    Reverse = "REVERSE"
    Trim = "TRIM"
    LTrim = "LTRIM"
    RTrim = "RTRIM"
    LPad = "LPAD"
    Locate = "LOCATE"
    Instr = "INSTR"
    Rand = "RAND"
    DateFormat = "DATE_FORMAT"
    DayOfWeek = "DAYOFWEEK"
    SecToTime = "SEC_TO_TIME"
    TimeDiff = "TIMEDIFF"
    TimeStamp = "TIMESTAMP"
    MD5 = "MD5"
    Cast = "CAST"
    Convert = "CONVERT"
    IsNull = "ISNULL"
    GroupConcat = "GROUP_CONCAT"
    Upper = "UPPER"
    Lower = "LOWER"
    Left = "LEFT"
    Right = "RIGHT"
    Abs = "ABS"
    Round = "ROUND"
    Ceiling = "CEILING"
    Floor = "FLOOR"
    Second = "SECOND"
    Minute = "MINUTE"
    Hour = "HOUR"
    Day = "DAY"
    Month = "MONTH"
    Year = "YEAR"
    Now = "NOW"
    Date = "DATE"
    DateDiff = "DATEDIFF"
    Coalesce = "COALESCE"
